package dsl.modelwithoutelements;

import java.util.Arrays;
import org.eclipse.viatra2.emf.incquery.runtime.api.IPatternMatch;
import org.eclipse.viatra2.emf.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.viatra2.emf.incquery.runtime.exception.IncQueryException;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.Pattern;

/**
 * Pattern-specific match representation of the modelWithoutElements pattern, 
 * to be used in conjunction with ModelWithoutElementsMatcher.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters, 
 * usable to represent a match of the pattern in the result of a query, 
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see ModelWithoutElementsMatcher
 * @see ModelWithoutElementsProcessor
 * 
 */
public final class ModelWithoutElementsMatch extends BasePatternMatch {
  private Object fM;
  
  private static String[] parameterNames = {"m"};
  
  ModelWithoutElementsMatch(final Object pM) {
    this.fM = pM;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("m".equals(parameterName)) return this.fM;
    return null;
    
  }
  
  public Object getM() {
    return this.fM;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if ("m".equals(parameterName) && newValue instanceof java.lang.Object) {
    	this.fM = (java.lang.Object) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setM(final Object pM) {
    this.fM = pM;
    
  }
  
  @Override
  public String patternName() {
    return "modelWithoutElements";
    
  }
  
  @Override
  public String[] parameterNames() {
    return ModelWithoutElementsMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fM};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"m\"=" + prettyPrintValue(fM));
    return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fM == null) ? 0 : fM.hashCode()); 
    return result; 
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (obj == null)
    	return false;
    if (!(obj instanceof IPatternMatch))
    	return false;
    IPatternMatch otherSig  = (IPatternMatch) obj;
    if (!pattern().equals(otherSig.pattern()))
    	return false;
    if (!ModelWithoutElementsMatch.class.equals(obj.getClass()))
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    ModelWithoutElementsMatch other = (ModelWithoutElementsMatch) obj;
    if (fM == null) {if (other.fM != null) return false;}
    else if (!fM.equals(other.fM)) return false;
    return true;
  }
  
  @Override
  public Pattern pattern() {
    try {
    	return ModelWithoutElementsMatcher.factory().getPattern();
    } catch (IncQueryException ex) {
     	// This cannot happen, as the match object can only be instantiated if the matcher factory exists
     	throw new IllegalStateException	(ex);
    }
    
  }
}
